/**
    @file ft51_usb.h

    @brief USB Slave Driver

    Definitions for USB Devices based on Universal Serial Bus Specification Revision 2.0
    from http://www.usb.org/developers/docs/

    @version 1.0.0

**/
/*
 * ============================================================================
 * History
 * =======
 *
 * (C) Copyright 2014, Future Technology Devices International Ltd.
 * ============================================================================
 *
 * This source code ("the Software") is provided by Future Technology Devices
 * International Limited ("FTDI") subject to the licence terms set out
 * http://www.ftdichip.com/FTSourceCodeLicenceTerms.htm ("the Licence Terms").
 * You must read the Licence Terms before downloading or using the Software.
 * By installing or using the Software you agree to the Licence Terms. If you
 * do not agree to the Licence Terms then do not download or use the Software.
 *
 * Without prejudice to the Licence Terms, here is a summary of some of the key
 * terms of the Licence Terms (and in the event of any conflict between this
 * summary and the Licence Terms then the text of the Licence Terms will
 * prevail).
 *
 * The Software is provided "as is".
 * There are no warranties (or similar) in relation to the quality of the
 * Software. You use it at your own risk.
 * The Software should not be used in, or for, any medical device, system or
 * appliance. There are exclusions of FTDI liability for certain types of loss
 * such as: special loss or damage; incidental loss or damage; indirect or
 * consequential loss or damage; loss of income; loss of business; loss of
 * profits; loss of revenue; loss of contracts; business interruption; loss of
 * the use of money or anticipated savings; loss of information; loss of
 * opportunity; loss of goodwill or reputation; and/or loss of, damage to or
 * corruption of data.
 * There is a monetary cap on FTDI's liability.
 * The Software may have subsequently been amended by another user and then
 * distributed by that other user ("Adapted Software").  If so that user may
 * have additional licence terms that apply to those amendments. However, FTDI
 * has no liability in relation to those amendments.
 * ============================================================================
 */

#ifndef __FT51_USB_H__
#define __FT51_USB_H__

#include <stdint.h>
#include "ft51.h"

/**
    @name USB Classes
    @brief Definitions of USB Class Codes.
        http://www.usb.org/developers/defined_class
 */
//@{
/// Use class information in the Interface Descriptors
#define USB_CLASS_DEVICE                        0x00
/// Audio
#define USB_CLASS_AUDIO                         0x01
/// Communications and CDC Control
#define USB_CLASS_CDC_CONTROL                   0x02
/// Human Interface Device
#define USB_CLASS_HID                           0x03
/// Physical
#define USB_CLASS_PHYSICAL                      0x05
/// Image
#define USB_CLASS_IMAGE                         0x06
/// Printer
#define USB_CLASS_PRINTER                       0x07
/// Mass Storage
#define USB_CLASS_MASS_STORAGE                  0x08
/// Hub
#define USB_CLASS_HUB                           0x09
/// CDC-Data
#define USB_CLASS_CDC_DATA                      0x0a
/// Smart Card
#define USB_CLASS_SMART_CARD                    0x0b
/// Content Security
#define USB_CLASS_CONTENT_SECURITY              0x0d
/// Video
#define USB_CLASS_VIDEO                         0x0e
/// Personal Healthcare
#define USB_CLASS_PERSONAL_HEALTHCARE           0x0f
/// Audio/Video Devices
#define USB_CLASS_AUDIO_VIDEO                   0x10
/// Diagnostic Device
#define USB_CLASS_DIAGNOSTIC_DEVICE             0xdc
/// Wireless Controller
#define USB_CLASS_WIRELESS_CONTROLLER           0xe0
/// Miscellaneous
#define USB_CLASS_MISCELLANEOUS                 0xef
/// Application Specific
#define USB_CLASS_APPLICATION                   0xfe
/// Vendor Specific
#define USB_CLASS_VENDOR                        0xff
//@}

/**
    @name USB Device Class
    @brief Definitions of USB SubClass and Protocols for Device Class.
        http://www.usb.org/developers/defined_class
 */
//@{
#define USB_SUBCLASS_DEVICE                     0x00
#define USB_PROTOCOL_DEVICE                     0x00
//@}

/**
    @name FTDI USB Information
    @brief Definitions of USB Vendor and Device IDs.
 */
//@{
/// FTDI Vendor ID
#define USB_VID_FTDI                            0x0403
/// FTDI Product ID for FT232 variants
#define USB_PID_FTDI_FT232                      0x6001
/// FTDI Product ID for FT2232 variants
#define USB_PID_FTDI_FT2232                     0x6010
/// FTDI Product ID for FT4232 variants
#define USB_PID_FTDI_FT4232                     0x6011
/// FTDI Product ID for FTX series
#define USB_PID_FTDI_FT_X_SERIES                0x6015
//@}

/**
    @name FTDI USB Version Information
    @brief Definitions of USB Version BCD codes.
 */
//@{
/// USB Version 2.0
#define USB_BCD_VERSION_2_0 0x0200 
//@}

/** 
    @name USB Standard Request Codes
    @brief Table 9-4. Standard Request Codes.
        Used as bRequest in USB_device_request and bDescriptorType 
        in USB_device_descriptor.
 */
//@{
/// 9.4.5 Get Status
#define USB_REQUEST_CODE_GET_STATUS                   0x00    
/// 9.4.1 Clear Feature
#define USB_REQUEST_CODE_CLEAR_FEATURE                0x01    
/// 9.4.9 Set Feature
#define USB_REQUEST_CODE_SET_FEATURE                  0x03    
/// 9.4.6 Set Address
#define USB_REQUEST_CODE_SET_ADDRESS                  0x05    
/// 9.4.3 Get Descriptor
#define USB_REQUEST_CODE_GET_DESCRIPTOR               0x06    
/// 9.4.8 Set Descriptor
#define USB_REQUEST_CODE_SET_DESCRIPTOR               0x07    
/// 9.4.2 Get Configuration
#define USB_REQUEST_CODE_GET_CONFIGURATION            0x08    
/// 9.4.7 Set Configuration
#define USB_REQUEST_CODE_SET_CONFIGURATION            0x09    
/// 9.4.4 Get Interface
#define USB_REQUEST_CODE_GET_INTERFACE                0x0A    
/// 9.4.10 Set Interface
#define USB_REQUEST_CODE_SET_INTERFACE                0x0B    
/// 9.4.11 Synch Frame
#define USB_REQUEST_CODE_SYNCH_FRAME                  0x0C    
//@}

/** 
    @name USB Standard Feature Selectors
    @brief Table 9-6 Standard Feature Selectors
    Used for wValue high byte in 9.4.1 Clear Feature, 9.4.9 Set Feature.
 */
//@{
/// Device
#define USB_FEATURE_DEVICE_REMOTE_WAKEUP 1
/// Endpoint
#define USB_FEATURE_ENDPOINT_HALT 0
/// Device
#define USB_FEATURE_TEST_MODE 2

/**
    @name USB Standard Get Status Response
    @brief Figure 9-4 Information Returned by a GetStatus() Request to a Device and
        Figure 9-6 Information Returned by a GetStatus() Request to an Endpoint
    Used for 2 bytes of data returned by Get Status request.
 */
//@{
#define USB_GET_STATUS_DEVICE_REMOTE_WAKEUP 2
#define USB_GET_STATUS_DEVICE_SELF_POWERED 1
#define USB_GET_STATUS_ENDPOINT_HALT 1

/**
    @name USB Descriptor Types
    @brief Table 9-5. Descriptor Types.
    Used for wValue high byte in 9.4.3 Get Descriptor, 9.4.8 Set Descriptor.
 */
//@{
/// 9.6.1 Device
#define USB_DESCRIPTOR_TYPE_DEVICE                    0x01
/// 9.6.3 Configuration
#define USB_DESCRIPTOR_TYPE_CONFIGURATION             0x02
/// 9.6.7 String
#define USB_DESCRIPTOR_TYPE_STRING                    0x03
/// 9.6.5 Interface
#define USB_DESCRIPTOR_TYPE_INTERFACE                 0x04
/// 9.6.6 Interface
#define USB_DESCRIPTOR_TYPE_ENDPOINT                  0x05
/// 9.6.2 Device Qualifier
#define USB_DESCRIPTOR_TYPE_DEVICE_QUALIFIER          0x06
/// 9.6.4 Other Speed Configuration
#define USB_DESCRIPTOR_TYPE_OTHER_SPEED_CONFIGURATION 0x07
#define USB_DESCRIPTOR_TYPE_INTERFACE_POWER           0x08
#define USB_DESCRIPTOR_TYPE_INTERFACE_ASSOCIATION     0x0B
//@}

/**
    @name USB bmRequestType Characteristics
    @brief Table 9-2. Format of Setup Data.
    Used for bmRequestType decoding.
 */
//@{
/// Data transfer direction
#define USB_BMREQUESTTYPE_DIR_MASK                    0x80
#define USB_BMREQUESTTYPE_DIR_HOST_TO_DEV             0x00
#define USB_BMREQUESTTYPE_DIR_DEV_TO_HOST             0x80
/// Transaction type
#define USB_BMREQUESTTYPE_TYPE_MASK                   0x60
#define USB_BMREQUESTTYPE_STANDARD                    0x00
#define USB_BMREQUESTTYPE_CLASS                       0x20
#define USB_BMREQUESTTYPE_VENDOR                      0x40
/// Transaction Recipient
#define USB_BMREQUESTTYPE_RECIPIENT_MASK              0x1F
#define USB_BMREQUESTTYPE_RECIPIENT_DEVICE            0x00
#define USB_BMREQUESTTYPE_RECIPIENT_INTERFACE         0x01
#define USB_BMREQUESTTYPE_RECIPIENT_ENDPOINT          0x02
//@}

/**
    @name Microsoft WCID Information
    @brief Definitions for WCID String Descriptors and vendor requests.
 **/
//@{
#define USB_MICROSOFT_WCID_VERSION 				 0x0100

#define USB_MICROSOFT_WCID_STRING_DESCRIPTOR	 0xEE
#define USB_MICROSOFT_WCID_STRING_LENGTH		 0x12
#define USB_MICROSOFT_WCID_STRING_SIGNATURE		 'M', 0x00, 'S', 0x00, \
	'F', 0x00, 'T', 0x00, '1', 0x00, '0', 0x00, '0', 0x00,

#define USB_MICROSOFT_WCID_STRING(A) \
		USB_MICROSOFT_WCID_STRING_LENGTH, \
		USB_DESCRIPTOR_TYPE_STRING, USB_MICROSOFT_WCID_STRING_SIGNATURE \
		A, 0x00,

#define USB_MICROSOFT_WCID_FEATURE_WINDEX_COMPAT_ID 0x0004
#define USB_MICROSOFT_WCID_FEATURE_WINDEX_DEVICE_GUID 0x0005

#define USB_MICROSOFT_WCID_PROPERTY_DATATYPE_STRING_SINGLE		0x1
#define USB_MICROSOFT_WCID_PROPERTY_DATATYPE_STRING_MULTI		0x7
//@}

/**
    @typedef USB_device_descriptor
    @brief Structure representing a USB Device Descriptor.
 **/
typedef struct USB_device_descriptor USB_device_descriptor;

/**
    @struct USB_device_descriptor
    @brief Structure representing a USB Device Descriptor. USB Spec Table 9-8.
 **/
struct USB_device_descriptor
{
    /** Size of descriptor in bytes. **/
    uint8_t  bLength;
    /** DEVICE Descriptor Type (always 1). **/
    uint8_t  bDescriptorType;
    /** USB Specification Release Number. **/
    uint16_t bcdUSB;
    /** USB Class code. **/
    uint8_t  bDeviceClass;
    /** USB Subclass code. **/
    uint8_t  bDeviceSubClass;
    /** USB Protocol code. **/
    uint8_t  bDeviceProtocol;
    /** Max packet size for EP 0. **/
    uint8_t  bMaxPacketSize0;
    /** Vendor ID. **/
    uint16_t idVendor;
    /** Product ID. **/
    uint16_t idProduct;
    /** Device release number. **/
    uint16_t bcdDevice;
    /** Index of manufacturer string descriptor. **/
    uint8_t  iManufacturer;
    /** Index of product string descriptor. **/
    uint8_t  iProduct;
    /** Index of serial no. string descriptor. **/
    uint8_t  iSerialNumber;
    /** Number of possible configurations. **/
    uint8_t  bNumConfigurations;
};

/**
    @typedef USB_configuration_descriptor
    @brief Structure representing a USB Configuration Descriptor.
 **/
typedef struct USB_configuration_descriptor USB_configuration_descriptor;

/**
    @struct USB_configuration_descriptor
    @brief Structure representing a USB Configuration Descriptor.
           USB Spec Table 9-10.
 **/
struct USB_configuration_descriptor
{
    /** Size of descriptor in bytes. **/
    uint8_t  bLength;
    /** CONFIGURATION descriptor type (always 2). **/
    uint8_t  bDescriptorType;
    /** Total length of bytes returned for this configuration. **/
    uint16_t wTotalLength;
    /** Number of interfaces supported by this configuration. **/
    uint8_t  bNumInterfaces;
    /** Argument for \p SetConfiguration() request. **/
    uint8_t  bConfigurationValue;
    /** Index of configuration string descriptor. **/
    uint8_t  iConfiguration;
    /** Configuration characteristics. **/
    uint8_t  bmAttributes;
    /** Max power consumption of the USB device. **/
    uint8_t  bMaxPower;
};

/**
    @name USB_config_descriptor bmAttributes
    @brief Config descriptor bmAttributes values.
 **/
//@{
#define USB_CONFIG_BMATTRIBUTES_REMOTE_WAKEUP     0x20
#define USB_CONFIG_BMATTRIBUTES_SELF_POWERED      0x40
#define USB_CONFIG_BMATTRIBUTES_RESERVED_SET_TO_1 0x80
//@}

/**
    @typedef USB_string_descriptor
    @brief Structure representing a USB String Descriptor.
 **/
typedef struct USB_string_descriptor USB_string_descriptor;

/**
    @struct USB_string_descriptor
    @brief Structure representing a USB String Descriptor.
           USB Spec Table 9-xx.
           Add array of unicode characters after structure to bLength
 **/
struct USB_string_descriptor
{
    /** Size of descriptor in bytes. **/
    uint8_t  bLength;
    /** STRING descriptor type (always 3). **/
    uint8_t  bDescriptorType;
};

/**
    @typedef USB_interface_descriptor
    @brief Structure representing a USB Interface Descriptor.
 **/
typedef struct USB_interface_descriptor USB_interface_descriptor;

/**
    @struct USB_interface_descriptor
    @brief Structure representing a USB Interface Descriptor.
           USB Spec Table 9-12.
 **/
struct USB_interface_descriptor
{
    /** Size of descriptor in bytes. **/
    uint8_t  bLength;
    /** INTERFACE descriptor type (always 4). **/
    uint8_t  bDescriptorType;
    /** Zero-base interface number. **/
    uint8_t  bInterfaceNumber;
    /** Alternate setting for this interface. **/
    uint8_t  bAlternateSetting;
    /** Number of endpoints in this interface. **/
    uint8_t  bNumEndpoints;
    /** Interface class. **/
    uint8_t  bInterfaceClass;
    /** Interface subclass. **/
    uint8_t  bInterfaceSubClass;
    /** Interface protocol. **/
    uint8_t  bInterfaceProtocol;
    /** Interface String Descriptor. **/
    uint8_t  iInterface;
};

/**
    @typedef USB_endpoint_descriptor
    @brief Structure representing a USB Endpoint Descriptor.
 **/
typedef struct USB_endpoint_descriptor USB_endpoint_descriptor;

/**
    @struct USB_endpoint_descriptor
    @brief Structure representing a USB Endpoint Descriptor.
           USB Spec Table 9-13.
 **/
struct USB_endpoint_descriptor
{
    /** Size of descriptor in bytes. **/
    uint8_t  bLength;
    /** ENDPOINT descriptor type (always 5). **/
    uint8_t  bDescriptorType;
    /** Endpoint address. **/
    uint8_t  bEndpointAddress;
    /** Endpoint attributes. **/
    uint8_t  bmAttributes;
    /** Endpoint maximum packet size. **/
    uint16_t wMaxPacketSize;
    /** Endpoint polling interval for interrupt endpoints. **/
    uint8_t  bInterval;
};

/**
    @name USB_config_descriptor bmAttributes
    @brief Config descriptor bmAttributes values.
 **/
//@{
#define USB_ENDPOINT_DESCRIPTOR_ATTR_MASK        0x03
#define USB_ENDPOINT_DESCRIPTOR_ATTR_CONTROL     0x00
#define USB_ENDPOINT_DESCRIPTOR_ATTR_ISOCHRONOUS 0x01
#define USB_ENDPOINT_DESCRIPTOR_ATTR_BULK        0x02
#define USB_ENDPOINT_DESCRIPTOR_ATTR_INTERRUPT   0x03
//@}

/**
    @typedef USB_device_request
    @brief Structure representing a USB Device Request.
 **/
typedef struct USB_device_request USB_device_request;

/**
    @struct USB_device_request
    @brief Structure representing a USB Device Request.  USB Spec Table 9-2.
 **/
struct USB_device_request
{
    /**< \verbatim
        Table 9-2
        D7: Data transfer direction
            0 = Host-to-device
            1 = Device-to-host
        D6...5: Type
            0 = Standard
            1 = Class
            2 = Vendor
            3 = Reserved
        D4...0: Recipient
            0 = Device
            1 = Interface
            2 = Endpoint
            3 = Other
            4...31 = Reserved
    \endverbatim*/
    uint8_t  bmRequestType;
    /** Specific request. **/
    uint8_t  bRequest;
    /** Varies according to request. **/
    uint16_t wValue;
    /** Varies according to request. **/
    uint16_t wIndex;
    /** Number of bytes to transfer for data stage. **/
    uint16_t wLength;
};

/**
    @struct Microsoft WCID Information
    @brief Definitions for WCID String Descriptors and vendor requests.
 **/
//@{
typedef struct USB_WCID_feature_descriptor
{
	/** The length, in bytes, of the complete extended compat ID descriptor. **/
	uint32_t dwLength;
	/** The descriptors version number, in binary coded decimal (BCD) format. **/
	uint16_t bcdVersion;
	/** An index that identifies the particular OS feature descriptor. **/
	uint16_t wIndex;
	/** The number of custom property sections. **/
	uint8_t bCount;
	/** reserved - 7 bytes. **/
	uint8_t rsv1[7];
	/** The interface or function number. **/
	uint8_t bFirstInterfaceNumber;
	/** reserved - 1 byte. **/
	uint8_t rsv2;
	/** The function's compatible ID. **/
	uint8_t compatibleID[8];
	/** The function's subcompatible ID. **/
	uint8_t subCompatibleID[8];
	/** reserved - 6 bytes. **/
	uint8_t rsv3[6];
} USB_WCID_feature_descriptor;

typedef struct USB_WCID_feature_descriptor_additional
{
	/** The interface or function number. **/
	uint8_t bFirstInterfaceNumber;
	/** reserved - 1 byte. **/
	uint8_t rsv2;
	/** The function's compatible ID. **/
	uint8_t compatibleID[8];
	/** The function's subcompatible ID. **/
	uint8_t subCompatibleID[8];
	/** reserved - 6 bytes. **/
	uint8_t rsv3[6];
} USB_WCID_feature_descriptor_additional;

typedef struct USB_WCID_extended_property_descriptor
{
	/** The length, in bytes, of the complete extended property (Header and Property sections) descriptor (146 bytes) **/
	uint32_t dwLength;
	/** The descriptors version number, in binary coded decimal (BCD) format. **/
	uint16_t bcdVersion;
	/** An index that identifies the particular OS feature descriptor. **/
	uint16_t wIndex;
	/** The number of custom property sections. **/
	uint16_t wCount;
	/** The Length of this custom property section  (136 bytes)  **/
	uint32_t dwSize;
	/** Property value stores a Unicode string **/
	uint32_t dwPropertyDataType;
	/** Length of the property name string  (42 bytes) **/
	uint16_t wPropertyNameLength;
	/** Property name is DeviceInterfaceGUIDs. Null terminated **/
	uint8_t bPropertyName[42];
	/** Length of the property value string (80 bytes) **/
	uint32_t dwPropertyDataLength;
	/** Property value in unicode e.g.{8FE6D4D7-49DD-41E7-9486-49AFC6BFE470}. Double Null termination **/
	uint8_t bPropertyData[80];
} USB_WCID_extended_property_descriptor;
//@}

/**
    @enum USB_STATE
    @brief USB States. USB Spec section 9.1.
 **/
typedef enum 
{
    /** Device is attached to USB. **/
    ATTACHED,
    /** Device is attached and powered. **/
    POWERED,
    /** Device is attached, has power and has been reset. **/
    DEFAULT,
    /** Unique device address has not been set. **/
    ADDRESS,
    /** Unique device address is now assigned.  Device can be used by host. **/
    CONFIGURED,
    /** Device has been suspended. **/
    SUSPENDED,
} USB_STATE;

/**
    @enum USB_ENDPOINT_NUMBER
    @brief USB Endpoint Numbers.
 **/
typedef enum
{
    /** Endpoint 0 (Control Endpoint) **/
    USB_EP_0,
    /** Endpoint 1. **/
    USB_EP_1,
    /** Endpoint 2. **/
    USB_EP_2,
    /** Endpoint 3. **/
    USB_EP_3,
    /** Endpoint 4. **/
    USB_EP_4,
    /** Endpoint 5. **/
    USB_EP_5,
    /** Endpoint 6. **/
    USB_EP_6,
    /** Endpoint 7. **/
    USB_EP_7
}
USB_ENDPOINT_NUMBER;

/**
    @enum USB_ENDPOINT_TYPE
    @brief USB Endpoint Types.
 **/
typedef enum 
{
    /** Control Endpoint. **/
    USB_EP_CTRL = 0,
    /** Bulk Endpoint. **/
    USB_EP_BULK = 1,
    /** Interrupt Endpoint. **/
    USB_EP_INT = 1,
    /** Isochronous Endpoint. **/
    USB_EP_ISOC = 2
} USB_ENDPOINT_TYPE;

/**
    @enum USB_ENDPOINT_DIR
    @brief USB Endpoint Direction.
 **/
typedef enum 
{
    /**Direction host to device. **/
    USB_DIR_OUT,
    /**Direction device to host. **/
    USB_DIR_IN,
} USB_ENDPOINT_DIR;

/**
    @enum USB_ENDPOINT_SIZE
    @brief USB Endpoint Sizes.
 **/
typedef enum 
{
    /** 8 Bytes **/
    USB_EP_SIZE_8   = 0x00,
    /** 16 Bytes **/
    USB_EP_SIZE_16  = 0x01,
    /** 32 Bytes **/
    USB_EP_SIZE_32  = 0x02,
    /** 64 Bytes **/
    USB_EP_SIZE_64  = 0x03,
    /** 96 Bytes.  Only available on \p ISO endpoints. **/
    USB_EP_SIZE_96  = 0x04,
    /** 128 Bytes.  Only available on \p ISO endpoints. **/
    USB_EP_SIZE_128 = 0x05,
    /** 160 Bytes.  Only available on \p ISO endpoints. **/
    USB_EP_SIZE_160 = 0x06,
    /** 192 Bytes.  Only available on \p ISO endpoints. **/
    USB_EP_SIZE_192 = 0x07,
    /** 256 Bytes.  Only available on \p ISO endpoints. **/
    USB_EP_SIZE_256 = 0x08,
    /** 320 Bytes.  Only available on \p ISO endpoints. **/
    USB_EP_SIZE_320 = 0x09,
    /** 384 Bytes.  Only available on \p ISO endpoints. **/
    USB_EP_SIZE_384 = 0x0a,
    /** 504 Bytes.  Only available on \p ISO endpoints. **/
    USB_EP_SIZE_504 = 0x0b,
} USB_ENDPOINT_SIZE;

/**
    @typedef   USB_request_callback
    @brief     Callback declaration for Vendor, Class and Standard USB 
               requests.
    @param[in] req USB request.
    @return    FT51_OK if the request was handled successfully; any other
               return value (such as FT51_FAILED) causes the USB driver
               to stall the control endpoints.
 **/
typedef FT51_STATUS (*USB_request_callback)(USB_device_request *req);

/**
    @typedef USB_suspend_callback
    @brief Callback declaration for a suspend/resume.
 **/
typedef void (*USB_suspend_callback)(uint8_t status);

/**
    @typedef USB_reset_callback
    @brief Callback declaration for a host reset.
 **/
typedef void (*USB_reset_callback)(uint8_t status);

/**
    @typedef USB_endpoint
    @brief Structure that represents a USB endpoint.
 **/
typedef struct USB_endpoint USB_endpoint;

/**
    @struct USB_endpoint
    @brief Structure that represents a USB endpoint.  This structure is used 
           in all calls to transfer data between the Host and Device.
 **/
struct USB_endpoint 
{
    /** Encodes USB endpoint number (0-7) and direction (In or Out). **/
    USB_ENDPOINT_NUMBER index;
    /** \p IN or \p OUT endpoint **/
    USB_ENDPOINT_DIR direction;
    /** Endpoint max packet size **/
    USB_ENDPOINT_SIZE max_packet_size;
    /** \p BULK, \p ISO, \p INT or \p CTRL endpoint **/
    USB_ENDPOINT_TYPE type;
    /**
        Status - see section 6.3.4 of the FT122 data sheet.\n
        Bit 0 and 1: Reserved.\n
        Bit 2: Indicates if last packet from host was a setup token (applies
               only to control endpoints).\n
        Bit 3 and 4: Reserved.\n
        Bit 5 and 6: Buffer 0 or 1 not empty.  For OUT endpoints, this
                     indicates unread data; for IN endpoints, this indicates
                     that you must wait for the host to issue another IN
                     request before you can use the endpoint again.\n
        Bit 7: Indicates if the endpoint has been stalled.
     **/
    uint8_t status;
    /**
        Last Transaction Status for endpoint.\n
        Bit 0: 0: indicates USB data receive or transmit not OK.\n
               1: indicates USB data receive or transmit OK.\n
        Bit 1, 2, 3, 4:
            0000: No error\n
            0001: PID encoding error\n
            0010: PID unknown\n
            0011: Unexpected packet\n
            0100: Token CRC error\n
            0101: Data CRC error\n
            0110: Time out error\n
            0111: Reserved\n
            1000: Enxpected EOP\n
            1001: Packet NAKed\n
            1010: Sent stall\n
            1011: Buffer overflow\n
            1101: Bit stuff error\n
            1111: Wrong DATA PID\n
        Bit 5: 0: indicates not a setup packet.\n
               1: indicates last received packet has a setup token.\n
        Bit 6: 0: packet has a DATA0 token.\n
               1: packet has a DATA1 token.\n
        Bit 7: 0: previous transaction status was read.\n
               1: previous transaction status was not read.\n
     **/
    uint8_t last_status;
};

/**
    @typedef USB_ctx
    @brief Struct containing callback functions for the USB upper layer 
           driver and callback functions for suspend/resume and reset.
 **/
typedef struct USB_ctx USB_ctx;

/**
    @struct USB_ctx
    @brief Struct containing callback functions for the USB upper layer 
           driver and callback functions for USB suspend/resume and 
           USB reset.  The struct also contains a flag to enable the USB hub.
 **/
struct USB_ctx
{
    /** Standard request callback function. **/
    USB_request_callback standard_req;
    /** [Optional] class request callback function. **/
    USB_request_callback class_req;
    /** [Optional] vendor request callback function. **/
    USB_request_callback vendor_req;
    /** [Optional] USB bus suspend callback function. **/
    USB_suspend_callback suspend;
    /** [Optional] USB bus suspend callback function. **/
    USB_suspend_callback resume;
    /** [Optional] USB bus LPM (Link Power Management) callback function. **/
    USB_suspend_callback lpm;
    /** [Optional] USB bus reset callback function. **/
    USB_reset_callback reset;

    /** Endpoint size for control endpoints. **/
    USB_ENDPOINT_SIZE ep_size;

    /** Hub configuration structure. **/
    union  {
        uint8_t value;
        struct {
            /** Alignment bit **/
            uint8_t hub_resv1:1;
            /** Hub Enable flag **/
            uint8_t hub_enable:1;
            /** Hub Self Powered flag **/
            uint8_t hub_LocalPwrSrc:1;
            /** Hub NAK Zero-length Data flag **/
            uint8_t hub_StsNzDataHsk:1;
            /** Hub Remote Wakeup flag **/
            uint8_t hub_RmtWkUpEn:1;
            /** Hub Compound Dev flag - must be 1 **/
            uint8_t hub_CompdDev:1;
        } mask;
    } hub_en;
};

/**
    @brief     \par Initialise USB hardware.

    Performs a software reset and intialises the USB hardware.\n
    The \p USB_ctx contains function pointers to protocol layer 
    handling of USB requests.  Appropriate USB requests will be 
    routed to the correct handler, whether that be Standard, Class 
    or Vendor requests.  A device may not need a handler for Vendor 
    or Class requests depending on the device configuration.\n\n
    Optional function pointers are also available for USB suspend/
    resume callbacks and bus resets issued by the host.\n
    This function MUST be called prior to any further call to the 
    USB functions.
    @param[in] ctx USB context.
    @return    N/A
 **/
void USB_initialise(USB_ctx *ctx);

/**
    @brief     \par Finalise USB hardware.

    Releases any resources associated with the USB driver and 
    disables the hardware.
    @return    Always FT51_OK.
 **/
void USB_finalise(void);

/**
    @brief     \par Set USB address.

    Places the device in the \p ADDRESS state.
    @param[in] req USB request.
    @return    \p FT51_OK on success.\n
               \p FT51_INVALID_PARAMETER if \p req is invalid.
 **/
FT51_STATUS USB_req_set_address(USB_device_request *req);

/**
    @brief     \par Set USB configuration.

    Places the device in the \p CONFIGURED state or puts it back 
    into the \p ADDRESS state.
    @param[in] req USB request.
    @return    \p FT51_OK on success.\n
               \p FT51_INVALID_PARAMETER if \p req is invalid.
 **/
FT51_STATUS USB_req_set_configuration(USB_device_request *req);

/**
    @brief     \par Get USB configuration.

    Returns the current configuration number.
    @return    \p FT51_OK on success.\n
               \p FT51_INVALID_PARAMETER if \p req is invalid.
 **/
FT51_STATUS USB_req_get_configuration(void);

/**
    @brief     \par Stall endpoint.

    Stalls the specified endpoint.
    @param[in] ep_number USB endpoint number.
    @param[in] ep_dir Endpoint direction, In or Out.
 **/
void USB_stall_endpoint(USB_ENDPOINT_NUMBER  ep_number,
        USB_ENDPOINT_DIR     ep_dir);

/**
    @brief     \par Clear endpoint.

    Clears a stall from the specified endpoint.
    @param[in] ep_number USB endpoint number.
    @param[in] ep_dir Endpoint direction, In or Out.
 **/
void USB_clear_endpoint(USB_ENDPOINT_NUMBER  ep_number,
        USB_ENDPOINT_DIR     ep_dir);

/**
    @brief      \par Get USB endpoint stall status.

    Returns the current stall status of an endpoint using the SELECT_ENDPOINT call.
    @param[in] ep_number USB endpoint number.
    @param[in] ep_dir Endpoint direction, In or Out.
    @return     Current stall state of the endpoint. TRUE if stalled, FALSE if not stalled.
 **/
uint8_t USB_get_ep_stalled(USB_ENDPOINT_NUMBER ep_number, USB_ENDPOINT_DIR ep_dir);

/**
    @brief      \par Get USB state.

    Returns the current state of the current USB device.  Please 
    refer to section 9.1 of the USB 2.0 spec for more information.
    @return     Current state of the current USB device.
 **/
USB_STATE USB_get_state(void);

/**
    @brief      \par Set USB state.

    Sets the current state of the current USB device.  Please 
    refer to section 9.1 of the USB 2.0 spec for more information.
    @param[in]  state The new state of the current USB device.
 **/
void USB_set_state(USB_STATE state);

/**
    @brief      \par Get USB endpoint buffer status.

    Returns the current buffer status of an endpoint using the SELECT_ENDPOINT call.
    @param[in] ep_number USB endpoint number.
    @param[in] ep_dir Endpoint direction, In or Out.
    @return     Current state of the endpoint buffer. TRUE if full, FALSE if empty.
 **/
uint8_t USB_ep_buffer_full(USB_ENDPOINT_NUMBER ep_number, USB_ENDPOINT_DIR ep_dir);

/**
    @brief      \par Get USB remote wakeup feature status.

    Returns the current feature status of remote wakeup.
    @return     Current remote wakeup feature status. TRUE if enabled, FALSE if not enabled.
 **/
uint8_t USB_get_remote_wakeup(void);

/**
    @brief      \par Set USB remote wakeup feature status.
 **/
void USB_set_remote_wakeup(void);

/**
    @brief      \par Clear USB remote wakeup feature status.
 **/
void USB_clear_remote_wakeup(void);

/**
    @brief      \par Resume with a USB remote wakeup.

    Perform a resume on the USB bus to initiate a remote wakeup when enabled.
 **/
void USB_wakeup(void);

/**
    @brief      \par Create a USB endpoint.

    Creates an endpoint with the requested properties.\n
    For \p OUT endpoints the \p out_callback function can be used 
    to notify the application when data is received from the host. 
    This function will be called from the endpoint interrupt 
    service routine.  If the value is \p NULL then the application 
    can poll the \p status.ep_full flag within the \p USB_endpoint 
    structure.\n
    @param[in] ep_number USB endpoint number.
    @param[in]  ep_type USB endpoint type: \p BULK, \p ISO or \p INT.
    @param[in] ep_dir Endpoint direction, In or Out.
    @param[in]  ep_size USB endpoint max packet size in bytes.
    @return     FT51_OK if sucessful. FT51_INVALID_PARAMETER if not.
 **/
FT51_STATUS USB_create_endpoint(USB_ENDPOINT_NUMBER   ep_num,
                                USB_ENDPOINT_TYPE     ep_type,
                                USB_ENDPOINT_DIR      ep_dir,
                                USB_ENDPOINT_SIZE     ep_size);

/**
    @brief      \par Free USB endpoint.

    Disable and free the specified endpoint.
    @param[in] ep_number USB endpoint number.
    @param[in] ep_dir Endpoint direction, In or Out.
    @return     N/A
 **/
void USB_free_endpoint(USB_ENDPOINT_NUMBER ep_number, USB_ENDPOINT_DIR ep_dir);

uint8_t USB_get_endpoint_status(USB_ENDPOINT_NUMBER ep_number, USB_ENDPOINT_DIR ep_dir);

/**
    @brief      \par Transfer data to/from a USB endpoint.

    USB \p IN or \p OUT request is implied from the \p ep 
    parameter.
    @param[in] ep_number USB endpoint number.
    @param[in] ep_dir Endpoint direction, In or Out.
    @param[in]  data Appropriately sized buffer for the transfer.
    @param[in]  length For \p IN transfers, the number of bytes to be sent.
                       For \p OUT transfers, the maximum number of bytes to 
                       read.
    @return     The number of bytes actually transferred.
 **/
uint16_t USB_transfer(USB_ENDPOINT_NUMBER ep_number, USB_ENDPOINT_DIR ep_dir,
                      uint8_t       *data,
                      const uint16_t length);
                      
/**
    @brief      \par USB process

    To be continuously called by user application. Checks for CONTROL transfer activity
    and invokes relevant callback.
    @param[in]  Nothing
    @return     Non-zero if USB transaction has been processed.
 **/                      
uint8_t USB_process(void);

/**
    @brief      \par USB Read LPM Status

    Reads LPM status, which is 9 bits of the bmAttributes of the LPM token; 
    (look at LPM ECN table 2-3)
    @param[in]  Nothing
    @return     uint16_t    LPM status
 **/
uint16_t USB_read_LPM_status(void);

/**
    @brief      \par Read frame number for last received SOF.
    
    @param[out]  frame_msb  Receives most-significant byte of frame number.
    @param[out]  frame_lsb  Receives least-significant byte of frame number.
 **/
void usb_read_frame_number(uint8_t *frame_msb,
                           uint8_t *frame_lsb);

/**
    @brief     Return the value of the HUB_ENABLE bit.

    This will return the suspend status of the hub suspend bit
    when the hub is enabled. If it is zero then the hub is suspended, it is
    non-zero then the hub is active.
    @note      Do not call this from an ISR.
**/
#define USB_GET_HUB_ENABLE(A) \
    do { \
        IO_REG_GENERAL_READ(TOP_USB_ENABLE, (A)); \
        (A) &= MASK_HUB_ENABLE; \
    } while (0)

#endif // __FT51_USB_H__
