/**
    @file ft51_usb_dfu.c

    @brief DFU Library API DFU functions.

    @version 1.0.0

**/
/*
 * ============================================================================
 * History
 * =======
 *
 * (C) Copyright 2014, Future Technology Devices International Ltd.
 * ============================================================================
 *
 * This source code ("the Software") is provided by Future Technology Devices
 * International Limited ("FTDI") subject to the licence terms set out
 * http://www.ftdichip.com/FTSourceCodeLicenceTerms.htm ("the Licence Terms").
 * You must read the Licence Terms before downloading or using the Software.
 * By installing or using the Software you agree to the Licence Terms. If you
 * do not agree to the Licence Terms then do not download or use the Software.
 *
 * Without prejudice to the Licence Terms, here is a summary of some of the key
 * terms of the Licence Terms (and in the event of any conflict between this
 * summary and the Licence Terms then the text of the Licence Terms will
 * prevail).
 *
 * The Software is provided "as is".
 * There are no warranties (or similar) in relation to the quality of the
 * Software. You use it at your own risk.
 * The Software should not be used in, or for, any medical device, system or
 * appliance. There are exclusions of FTDI liability for certain types of loss
 * such as: special loss or damage; incidental loss or damage; indirect or
 * consequential loss or damage; loss of income; loss of business; loss of
 * profits; loss of revenue; loss of contracts; business interruption; loss of
 * the use of money or anticipated savings; loss of information; loss of
 * opportunity; loss of goodwill or reputation; and/or loss of, damage to or
 * corruption of data.
 * There is a monetary cap on FTDI's liability.
 * The Software may have subsequently been amended by another user and then
 * distributed by that other user ("Adapted Software").  If so that user may
 * have additional licence terms that apply to those amendments. However, FTDI
 * has no liability in relation to those amendments.
 * ============================================================================
 */
#include <stdlib.h>

#include "ft51_usb.h"
#include "ft51_usb_dfu.h"
#include "ft51_io_registers.h"
#include "registers/ft51_mtp_registers.h"

#include "ft51_dfu.h"

/**
 @name  detach_counter
 @brief Countdown millisecond counter to return to DFUSTATE_APPIDLE
        from DFUSTATE_APPATTACH if a reset is not received within the
        time period specified in DFU_DETACH request.
 */
static volatile uint16_t detach_counter = 0;

/**
 @name  dfuState
 @brief Current state machine position as defined in enum DFU_STATE.
 */
static uint8_t dfuState = DFUSTATE_APPIDLE;

/**
 @name  dfuStatus
 @brief Current state machine status as defined in enum DFU_STATUS.
        This can be DFUSTATUS_OK or an error state.
 */
static uint8_t dfuStatus = DFUSTATUS_OK;


unsigned char dfu_is_runtime(void)
{
    return (dfuState < DFUSTATE_DFUIDLE);
}

unsigned char dfu_is_wait_reset(void)
{
    return (dfuState == DFUSTATE_MANIFEST_WAIT_RESET);
}

void dfu_class_req_detach(uint16_t timeout)
{

    if (dfuState == DFUSTATE_APPIDLE)
    {
        dfuState = DFUSTATE_APPDETACH;
        // Initialise timeout timer
        detach_counter = timeout;
    }

    /* Respond with an ACK. */
    USB_transfer(USB_EP_0, USB_DIR_IN, NULL, 0);

    return;
}

void dfu_class_req_download(uint16_t address, uint16_t length)
{
    uint8_t buf[DFU_MAX_BLOCK_SIZE];
    FT51_STATUS status = FT51_OK;
    uint8_t written;
    uint8_t *pbuf;

    if (length == 0)
    {
        //USB_ack_setup(ep_ctrl_out, ep_ctrl_in);

        if (dfuState == DFUSTATE_DNLOAD_IDLE)
        {
            // Transfer finishes with a zero-length packet.
            dfuState = DFUSTATE_MANIFEST_SYNC;
        }
    }
    else if (length <= DFU_MAX_BLOCK_SIZE)
    {
        // Read in data from OUT endpoint.
        USB_transfer(USB_EP_0, USB_DIR_OUT, buf, length);

        // Now change state machine to downloading.
        dfuState = DFUSTATE_DNLOAD_BUSY;

        // Prevent writes above the top of MTP memory.
        if ((address + length) <= DFU_MTP_TOP_ADDRESS)
        {
            // Write each byte into MTP.
            pbuf = buf;

            while (length--)
            {
                dfuState = DFUSTATE_DNLOAD_BUSY;
                // Program byte into MTP
                IO_REG_GENERAL_WRITE(MTP_ADDR_L, (address & 0xff));
                IO_REG_GENERAL_WRITE(MTP_ADDR_U, ((address >> 8) & 0xff));
                IO_REG_GENERAL_WRITE(MTP_PROG_DATA, (*pbuf));

                // Read the MTP_CONTROL register repeatedly for up to
                // 10 milliseconds until PROG_DONE is asserted.
                detach_counter = 10;

                do
                {
                    IO_REG_GENERAL_READ(MTP_CONTROL, written);

                    // MTP byte program completed
                    if (written & MASK_MTP_BYTE_PROG_DONE)
                    {
                        dfuState = DFUSTATE_DNLOAD_SYNC;
                        break;
                    }

                } while (detach_counter > 0);

                if (detach_counter == 0)
                {
                    // MTP byte program timeout.
                    dfuStatus = DFUSTATUS_ERR_NOTDONE;
                    status = FT51_FAILED;
                    break;
                }
                pbuf++;
                address++;
            }
        }
        else
        {
            // Attempt to write past top block of MTP.
            dfuStatus = DFUSTATUS_ERR_ADDRESS;
            status = FT51_FAILED;
        }
    }
    else
    {
        // Attempt to write more than DFU_MAX_BLOCK_SIZE bytes at a time.
        dfuStatus = DFUSTATUS_ERR_WRITE;
        status = FT51_FAILED;
    }

    // ACK SETUP only if successful.
    if (status == FT51_OK)
    {
        USB_transfer(USB_EP_0, USB_DIR_IN, NULL, 0);
    }
    // If failure then STALL.
    else
    {
        USB_stall_endpoint(USB_EP_0, USB_DIR_OUT);
        dfuState = DFUSTATE_DFUERROR;
    }

    return;
}

void dfu_class_req_getstatus(void)
{
    // space to hold our response...
    dfu_status_t buf;

    buf.bStatus = dfuStatus;
    buf.bwPollTimeout[0] = 0x01;
    buf.bwPollTimeout[1] = 0x00;
    buf.bwPollTimeout[2] = 0x00;
    buf.bState = dfuState;
    buf.iString = 0x00;

    USB_transfer(USB_EP_0, USB_DIR_IN, (unsigned char *) &buf, sizeof(dfu_status_t));

    if (dfuState == DFUSTATE_DNLOAD_SYNC)
    {
        dfuState = DFUSTATE_DNLOAD_IDLE;
    }
    else if (dfuState == DFUSTATE_MANIFEST_SYNC)
    {
        dfuState = DFUSTATE_MANIFEST_WAIT_RESET;
    }

    return;
}

void dfu_class_req_clrstatus(void)
{
    if (dfuState == DFUSTATE_DFUERROR)
    {
        dfuState = DFUSTATE_DFUIDLE;
    }
    dfuStatus = DFUSTATUS_OK;

    /* Respond with an ACK. */
    USB_transfer(USB_EP_0, USB_DIR_IN, NULL, 0);

    return;
}

void dfu_class_req_abort(void)
{
    if (dfuState > DFUSTATE_DFUIDLE)
    {
        dfuState = DFUSTATE_DFUIDLE;
    }

    /* Respond with an ACK. */
    USB_transfer(USB_EP_0, USB_DIR_IN, NULL, 0);

    return;
}

void dfu_class_req_getstate(void)
{
    // space to hold our response...
    unsigned char bState;

    bState = dfuState;

    USB_transfer(USB_EP_0, USB_DIR_IN, &bState, sizeof(bState));

    return;
}

unsigned char dfu_reset(void)
{
    if (dfuState == DFUSTATE_APPDETACH)
    {
        // Move into DFU mode.
        dfuState = DFUSTATE_DFUIDLE;
    }
    else if (dfuState == DFUSTATE_MANIFEST_WAIT_RESET)
    {
        // Back to application mode and load new firmware.
        dfuState = DFUSTATE_APPIDLE;
        return 1;
    }
    else if (dfuState == DFUSTATE_DNLOAD_IDLE)
    {
        // Incomplete firmware download, report error, do not reload firmware.
        dfuState = DFUSTATE_DFUERROR;
    }
    //else if (dfuState == DFUSTATE_DFUIDLE)
    //{
        // Back to starting mode (no reload of firmware).
        // NOTE: This transition is disabled as ft51dfu application (libusb)
        // sends multiple USB bus resets on a usb_reset_ex() call. On a detach
        // This will transition from DFUSTATE_APPIDLE to DFUSTATE_DFUIDLE and
        // back to DFUSTATE_APPIDLE thus preventing DFU mode from being
        // entered.
        //dfuState = DFUSTATE_APPIDLE;
    //}
    // If state is DFUSTATE_DFUERROR then remain in that state until
    // successful firmware download occurs.

    return 0;
}

unsigned char dfu_timer(void)
{
    unsigned char ret_val = 0;

    if (detach_counter)
    {
        detach_counter--;

        if (detach_counter == 0)
        {
            // if detach counter expires then move back to idle state
            if (dfuState == DFUSTATE_APPDETACH)
            {
                dfuState = DFUSTATE_APPIDLE;
                ret_val = 1;
            }
        }
    }
    return ret_val;
}
