/**
    @file ft51_usb_endpoint.c

    @brief USB Library API endpoint functions.

    @version 1.0.0

**/
/*
 * ============================================================================
 * History
 * =======
 *
 * (C) Copyright 2014, Future Technology Devices International Ltd.
 * ============================================================================
 *
 * This source code ("the Software") is provided by Future Technology Devices
 * International Limited ("FTDI") subject to the licence terms set out
 * http://www.ftdichip.com/FTSourceCodeLicenceTerms.htm ("the Licence Terms").
 * You must read the Licence Terms before downloading or using the Software.
 * By installing or using the Software you agree to the Licence Terms. If you
 * do not agree to the Licence Terms then do not download or use the Software.
 *
 * Without prejudice to the Licence Terms, here is a summary of some of the key
 * terms of the Licence Terms (and in the event of any conflict between this
 * summary and the Licence Terms then the text of the Licence Terms will
 * prevail).
 *
 * The Software is provided "as is".
 * There are no warranties (or similar) in relation to the quality of the
 * Software. You use it at your own risk.
 * The Software should not be used in, or for, any medical device, system or
 * appliance. There are exclusions of FTDI liability for certain types of loss
 * such as: special loss or damage; incidental loss or damage; indirect or
 * consequential loss or damage; loss of income; loss of business; loss of
 * profits; loss of revenue; loss of contracts; business interruption; loss of
 * the use of money or anticipated savings; loss of information; loss of
 * opportunity; loss of goodwill or reputation; and/or loss of, damage to or
 * corruption of data.
 * There is a monetary cap on FTDI's liability.
 * The Software may have subsequently been amended by another user and then
 * distributed by that other user ("Adapted Software").  If so that user may
 * have additional licence terms that apply to those amendments. However, FTDI
 * has no liability in relation to those amendments.
 * ============================================================================
 */
#include <stdint.h>
#include <stdlib.h>
#include <string.h>
#include "ft51.h"
#include "ft51_sfr.h"
#include "ft51_usb.h"
#include "ft51_usb_internal.h"

#ifndef USB_ENDPOINT_CHECKS
#define USB_ENDPOINT_CHECKS 0
#endif // USB_ENDPOINT_CHECKS


#if USB_ENDPOINT_CHECKS > 0
/**
 @brief     \par Check Endpoint Memory Free
 @details   Checks how much memory is left in the endpoint buffers. See
 section 5.2 of the FT122 Datasheet.
 @param[in] ep_dir The endpoint direction.
 @return    The number of bytes free.
 **/
uint16_t USB_get_ep_memory_free(const USB_ENDPOINT_DIR ep_dir)
{
    uint8_t i = 0;
    uint16_t total = 0;

    for (i = 0; i < _countof(USB_ep); i++)
    {
        USB_endpoint *ep = &USB_ep[i];

        if (ep == NULL)
            continue;

        if (ep->direction == ep_dir)
            total += USB_ep_size_bytes[ep->max_packet_size];
    }

    return 1024 - total;
}
#endif // USB_ENDPOINT_CHECKS


/**
 @brief     \par Create an Endpoint Handle
 @details   Initialises the memory associated with an endpoint
 handle and enables the endpoint in the FT12x device.
 @param[in] ep_num The endpoint number.
 @param[in] ep_type The endpoint type.
 @param[in] ep_dir The endpoint direction.
 @param[in] ep_size The endpoint max packet packet size.
 @return    FT51_OK if sucessful. FT51_INVALID_PARAMETER if not.
 **/
FT51_STATUS USB_create_endpoint(USB_ENDPOINT_NUMBER ep_num,
                                USB_ENDPOINT_TYPE ep_type,
                                USB_ENDPOINT_DIR ep_dir,
                                USB_ENDPOINT_SIZE ep_size)
{
    // Refer to section 5.2 of the FT122 datasheet for information on the
    // parameters to this cmd...
    // Refer to section 6.2.5 for information on obtaining endpoints.
    uint8_t ep_data = 0x00;
    USB_endpoint *new_ep;
    uint8_t ep_index = (ep_num * 2) + ep_dir;

#if USB_ENDPOINT_CHECKS > 0
    if (ep_index >= USB_MAX_ENDPOINT_COUNT)
        return FT51_INVALID_PARAMETER; // EP number higher than maximium allowed.

    if (ep_type != USB_EP_ISOC && ep_size > USB_EP_SIZE_64)
        return FT51_INVALID_PARAMETER;// EP Size too big.

    if (USB_ep[ep_index])
        return FT51_INVALID_PARAMETER;// Endpoint already in use.

    // Check there is enough memory for the EP size requested...
    if (USB_get_ep_memory_free(ep_dir) < ep_size)
        return FT51_INVALID_PARAMETER;// Not enough memory in EP buffers.

#endif // USB_ENDPOINT_CHECKS

    new_ep = &USB_ep[ep_index];

    // Set EP cmd parameters...
    ep_data |= 0x01; // Turn on enable bit.
    ep_data |= (ep_type << 1); // Mask in ep type.
    ep_data |= (ep_size << 3); // Mask in the ep_size param.

    DISABLE_INTERRUPTS;
    // Configure this endpoint
    FT122_CMD = SET_ENDPOINT_CONFIGURATION + ep_index;
    FT122_DATA = ep_data;
    // Enable ALL end-points
    FT122_CMD = SET_ENDPOINT_ENABLE;
    FT122_DATA = 1;
    ENABLE_INTERRUPTS;

    // All parameters are correct, go on to configure the endpoint...
    new_ep->index = ep_index;
    new_ep->direction = ep_dir;
    new_ep->max_packet_size = ep_size;
    new_ep->type = ep_type;

    return FT51_OK;
}

/**
 @brief     \par Releases an Endpoint Handle
 @details   Clears the memory associated with an endpoint
 handle and disables the endpoint in the FT12x device.
 @param[in] ep_number The endpoint number.
 @param[in] ep_dir The endpoint direction.
 **/
void USB_free_endpoint(USB_ENDPOINT_NUMBER ep_number, USB_ENDPOINT_DIR ep_dir)
{
    uint8_t ep_index = (ep_number * 2) + ep_dir;

    DISABLE_INTERRUPTS;
    // Disable endpoint...
    FT122_CMD = SET_ENDPOINT_CONFIGURATION + ep_index;
    FT122_DATA = 0x00;
    ENABLE_INTERRUPTS;

    memset(&USB_ep[ep_index], 0, sizeof(USB_endpoint));
}